@@ -18,6 +18,7 @@ import functools
 import hashlib
 import numpy as np
 import pathlib
+import sqlite3
 
 import opentimspy
 
@@ -45,6 +46,7 @@ class OpenTIMS:
             Args:
                 analysis_directory (str, unicode string): path to the folder containing 'analysis.tdf' and 'analysis.tdf_raw'.
         """
+        self.handle = None
         self.analysis_directory = pathlib.Path(analysis_directory)
         self.handle = opentimspy.opentimspy_cpp.TimsDataHandle(str(analysis_directory))
         self.frames = self.table2dict("Frames")
@@ -93,8 +95,21 @@ class OpenTIMS:
 
 
     def __del__ (self):
-        if hasattr(self, 'handle'):
+        self.close()
+
+    def close(self):
+        if not self.handle is None:
             del self.handle
+            self.handle = None
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
+    def get_sql_connection(self):
+        return sqlite3.connect(self.analysis_directory/"analysis.tdf")
 
     def tables_names(self):
         return tables_names(self.analysis_directory/"analysis.tdf")
@@ -353,6 +368,179 @@ class OpenTIMS:
                                                       self.max_frame+1),
                                          columns=columns)]
 
+
+    def retention_time_to_frame(
+        self,
+        retention_time: np.array,
+        _buffer=1,
+    ) -> np.array:
+        """Transform retention times into their corresponding frame numbers.
+
+        We check if retention times are within sensible bounds.
+
+        Arguments:
+            retention_time (np.array): An array of retention times.
+
+        Returns:
+            np.array: integers, numbers of respective frames (Tims pushes).
+        """
+        retention_time = np.array(retention_time)# if someone passes a float or a pandas.Series
+        assert all(retention_time <= self.max_retention_time + _buffer), "Some retention times were higher than the latest one."
+        res = np.searchsorted(self.retention_times, retention_time)
+        return res + 1
+
+
+    def MS1_retention_time_to_frame(
+        self,
+        retention_time: np.array,
+        _buffer=1,
+    ) -> np.array:
+        """Transform MS1 retention times into their corresponding frame numbers.
+
+        This is to be sure that we get values only in MS1.
+        We check if retention times are within sensible bounds.
+
+        Arguments:
+            retention_time (np.array): An array of retention times.
+        Returns:
+            np.array: integers, numbers of respective frames (Tims pushes).
+        """
+        retention_time = np.array(retention_time)# if someone passes a float or a pandas.Series
+        all_ms1_rts = self.retention_times[self.ms1_frames - 1]
+        assert all(retention_time <= all_ms1_rts[-1] + _buffer), "Some retention times were higher than the last MS1 one."
+        res = np.searchsorted(all_ms1_rts, retention_time)
+        return self.ms1_frames[res]
+
+
+    def frame_to_retention_time(
+        self,
+        frame: np.array
+    ) -> np.array:
+        """Transform frames into their corresponding retention times.
+
+        We check if frames are within sensible bounds.
+
+        Arguments:
+            frame (np.array): An array of frames.
+
+        Returns:
+            np.array: retention time when a frame finishes [second].
+        """
+        assert all(frame >= self.min_frame), "Some frames were below the minimal one."
+        assert all(frame <= self.max_frame), "Some frames were above the maximal one."
+        return self.retention_times[frame-1]
+
+
+    def scan_to_inv_ion_mobility(
+        self, 
+        scan: np.array,
+        frame: np.array,
+    ) -> np.array:
+        """Transform scans into their corresponding inverse ion mobilities.
+
+        We check if scans are within sensible bounds.
+
+        Arguments:
+            frame (np.array): An array of integer scans.
+
+        Returns:
+            np.array: inverse ion mobilities [1/k0].
+        """
+        scan = np.array(scan, dtype=np.uint32)
+        assert all(scan >= self.min_scan), "Some scans were below the minimal one."
+        assert all(scan <= self.max_scan), "Some scans were above the maximal one."
+        assert all(frame >= self.min_frame), "Some frames were below the minimal one."
+        assert all(frame <= self.max_frame), "Some frames were above the maximal one."
+        inv_ion_mobility = np.empty(len(scan), dtype=np.double)
+        for frame_id in np.unique(frame):
+            inv_ion_mobility[frame == frame_id] = self.handle.scan_to_inv_mobility(
+                frame_id,
+                scan[frame == frame_id]
+            )
+        return inv_ion_mobility
+
+
+    def inv_ion_mobility_to_scan(
+        self,
+        inv_ion_mobility: np.array,
+        frame: np.array,
+        _buffer: float = 0.0,
+    ) -> np.array:
+        """Transform inverse ion mobilities into their corresponding scan values.
+
+        We check if scans are within sensible bounds.
+        Scans correspond to individual emptyings of the second TIMS trap.
+
+        Arguments:
+            frame (np.array): An array of integer scans.
+
+        Returns:
+            np.array: inverse ion mobilities [1/k0].
+        """
+        inv_ion_mobility = np.array(inv_ion_mobility, dtype=np.double)
+        assert all(inv_ion_mobility >= self.min_inv_ion_mobility - _buffer), "Some inverse ion mobilities were below the minimal one."
+        assert all(inv_ion_mobility <= self.max_inv_ion_mobility + _buffer), "Some inverse ion mobilities were above the maximal one."
+        assert all(frame >= self.min_frame), "Some frames were below the minimal one."
+        assert all(frame <= self.max_frame), "Some frames were above the maximal one."
+        scan = np.empty(len(inv_ion_mobility), dtype=np.uint32)
+        for frame_id in np.unique(frame):
+            scan[frame == frame_id] = self.handle.inv_mobility_to_scan(
+                frame_id, 
+                inv_ion_mobility[frame == frame_id]
+            )
+        return scan
+
+
+    def tof_to_mz(self, tof: np.array, frame: np.array):
+        """Transform time of flight indices (tof) into their corresponding mass to charge ratios (m/z).
+
+        Caution!
+        We do not check if the values are sensible, i.e. if they correspond to meaningful outputs.
+
+        Arguments:
+            tof (np.array): An array of time of flight integers.
+
+        Returns:
+            np.array: array of doubles with m/z values.
+        """
+        tof = np.array(tof, dtype=np.uint32)
+        mz = np.empty(len(tof), dtype=np.double)
+        assert all(frame >= self.min_frame), "Some frames were below the minimal one."
+        assert all(frame <= self.max_frame), "Some frames were above the maximal one."
+        for frame_id in np.unique(frame):
+            mz[frame == frame_id] = self.handle.tof_to_mz(frame_id, tof[frame == frame_id])
+        return mz
+
+
+    def mz_to_tof(
+        self,
+        mz: np.array,
+        frame: np.array,
+        _buffer: float=0.0,
+    ):
+        """Transform mass to charge ratios (m/z) into their corresponding time of flight indices (tof).
+
+        We check if m/z values are within sensible bounds.
+        Time of flight indices are somehow proportional to time of flights.
+        We are figuring out how.
+        
+        Arguments:
+            mz (np.array): An array of m/z floats.
+
+        Returns:
+            np.array: integer time of flight indices.
+        """
+        mz = np.array(mz, dtype=np.double)
+        assert all(mz >= self.min_mz - _buffer), "Some m/z values were below the minimal one."
+        assert all(mz <= self.max_mz + _buffer), "Some m/z values were above the maximal one."
+        assert all(frame >= self.min_frame), "Some frames were below the minimal one."
+        assert all(frame <= self.max_frame), "Some frames were above the maximal one."
+        tof = np.empty(len(mz), dtype=np.uint32)
+        for frame_id in np.unique(frame):
+            tof[frame == frame_id] = self.handle.mz_to_tof(frame_id, mz[frame == frame_id])
+        return tof
+
+
     @functools.lru_cache(maxsize=1)
     def framesTIC(self):
         """Get the Total Ion Current for each frame.
