@@ -22,27 +22,32 @@
 #include "opentimsr_types.h"
 
 // adding default converters.
-#include "scan2drift_converter.h"
-#include "tof2mz_converter.h"
+#include "converters.h"
+#include "thread_mgr.h"
 
 
 // [[Rcpp::export(.setup_bruker_so)]]
 void setup_bruker_so(const Rcpp::String& path)
 {
-    DefaultTof2MzConverterFactory::setAsDefault<BrukerTof2MzConverterFactory, const char*>(path.get_cstring());
-    DefaultScan2DriftConverterFactory::setAsDefault<BrukerScan2DriftConverterFactory, const char*>(path.get_cstring());
+    setup_bruker(std::string(path.get_cstring()));
 }
 
 
 // [[Rcpp::export]]
-Rcpp::XPtr<TimsDataHandle> tdf_open(const Rcpp::String& path_d)
+Rcpp::XPtr<TimsDataHandle> tdf_open(const Rcpp::String& path_d, const Rcpp::List& sql_res)
 {
     TimsDataHandle* p; 
-    p = new TimsDataHandle(path_d);
+    p = new TimsDataHandle(path_d, sql_res);
     return Rcpp::XPtr<TimsDataHandle>(p, true);
 }
 
 
+// [[Rcpp::export]]
+void tdf_close(Rcpp::XPtr<TimsDataHandle> tdf)
+{
+    tdf.release();
+}
+
 
 // [[Rcpp::export]]
 uint32_t tdf_min_frame_id(Rcpp::XPtr<TimsDataHandle> tdf)
@@ -65,25 +70,6 @@ size_t tdf_no_peaks_total(Rcpp::XPtr<TimsDataHandle> tdf)
 }
 
 
-// [[Rcpp::export]]
-Rcpp::IntegerVector tdf_get_msms_type(Rcpp::XPtr<TimsDataHandle> tdf)
-{
-    using namespace Rcpp;
-    size_t start = tdf->min_frame_id();
-    size_t end = tdf->min_frame_id();
-    int n = end - start;
-
-    TimsDataHandle& tdh = *tdf;
-    IntegerVector msms_types(n);
-
-    for(size_t idx = start; idx < end; idx += 1)
-    {
-
-    }
-
-    return msms_types;
-} 
-
 // [[Rcpp::export]]
 Rcpp::DataFrame tdf_get_range(Rcpp::XPtr<TimsDataHandle> tdf, size_t start, size_t end, int32_t step = 1)
 {
@@ -196,8 +182,8 @@ Rcpp::DataFrame tdf_extract_frames(
     const bool get_tofs = true,
     const bool get_intensities = true,
     const bool get_mzs = true,
-    const bool get_dts = true,
-    const bool get_rts = true)
+    const bool get_inv_ion_mobilities = true,
+    const bool get_retention_times = true)
 {
     using namespace Rcpp;
 
@@ -214,8 +200,8 @@ Rcpp::DataFrame tdf_extract_frames(
     std::unique_ptr<uint32_t[]> tofs = R_get_ptr<uint32_t>(peaks_no, true);
     std::unique_ptr<uint32_t[]> intensities = R_get_ptr<uint32_t>(peaks_no, true);
     std::unique_ptr<double[]> mzs = R_get_ptr<double>(peaks_no, get_mzs);
-    std::unique_ptr<double[]> dts = R_get_ptr<double>(peaks_no, get_dts);
-    std::unique_ptr<double[]> rts = R_get_ptr<double>(peaks_no, get_rts);
+    std::unique_ptr<double[]> inv_ion_mobilities = R_get_ptr<double>(peaks_no, get_inv_ion_mobilities);
+    std::unique_ptr<double[]> retention_times = R_get_ptr<double>(peaks_no, get_retention_times);
 
     tdh.extract_frames(
         v.get(),
@@ -225,8 +211,8 @@ Rcpp::DataFrame tdf_extract_frames(
         tofs.get(),
         intensities.get(),
         mzs.get(),
-        dts.get(),
-        rts.get()
+        inv_ion_mobilities.get(),
+        retention_times.get()
     );
 
     DataFrame result = DataFrame::create();
@@ -237,8 +223,8 @@ Rcpp::DataFrame tdf_extract_frames(
     set_frame<uint32_t, Rcpp::IntegerVector>(result, "intensity", intensities, peaks_no);
     
     set_frame<double, Rcpp::NumericVector>(result, "mz", mzs, peaks_no);
-    set_frame<double, Rcpp::NumericVector>(result, "dt", dts, peaks_no);
-    set_frame<double, Rcpp::NumericVector>(result, "rt", rts, peaks_no);
+    set_frame<double, Rcpp::NumericVector>(result, "inv_ion_mobility", inv_ion_mobilities, peaks_no);
+    set_frame<double, Rcpp::NumericVector>(result, "retention_time", retention_times, peaks_no);
 
     return result;
 }
@@ -256,8 +242,8 @@ Rcpp::DataFrame tdf_extract_frames_slice(
     const bool get_tofs = true,
     const bool get_intensities = true,
     const bool get_mzs = true,
-    const bool get_dts = true,
-    const bool get_rts = true)
+    const bool get_inv_ion_mobilities = true,
+    const bool get_retention_times = true)
 {
     using namespace Rcpp;
 
@@ -271,8 +257,8 @@ Rcpp::DataFrame tdf_extract_frames_slice(
     std::unique_ptr<uint32_t[]> tofs = R_get_ptr<uint32_t>(peaks_no, true);
     std::unique_ptr<uint32_t[]> intensities = R_get_ptr<uint32_t>(peaks_no, true);
     std::unique_ptr<double[]> mzs = R_get_ptr<double>(peaks_no, get_mzs);
-    std::unique_ptr<double[]> dts = R_get_ptr<double>(peaks_no, get_dts);
-    std::unique_ptr<double[]> rts = R_get_ptr<double>(peaks_no, get_rts);
+    std::unique_ptr<double[]> inv_ion_mobilities = R_get_ptr<double>(peaks_no, get_inv_ion_mobilities);
+    std::unique_ptr<double[]> retention_times = R_get_ptr<double>(peaks_no, get_retention_times);
 
     tdh.extract_frames_slice(
         start, 
@@ -283,8 +269,8 @@ Rcpp::DataFrame tdf_extract_frames_slice(
         tofs.get(),
         intensities.get(),
         mzs.get(),
-        dts.get(),
-        rts.get()
+        inv_ion_mobilities.get(),
+        retention_times.get()
     );
 
     DataFrame result = DataFrame::create();
@@ -295,8 +281,14 @@ Rcpp::DataFrame tdf_extract_frames_slice(
     set_frame<uint32_t, Rcpp::IntegerVector>(result, "intensity", intensities, peaks_no);
     
     set_frame<double, Rcpp::NumericVector>(result, "mz", mzs, peaks_no);
-    set_frame<double, Rcpp::NumericVector>(result, "dt", dts, peaks_no);
-    set_frame<double, Rcpp::NumericVector>(result, "rt", rts, peaks_no);
+    set_frame<double, Rcpp::NumericVector>(result, "inv_ion_mobility", inv_ion_mobilities, peaks_no);
+    set_frame<double, Rcpp::NumericVector>(result, "retention_time", retention_times, peaks_no);
 
     return result;
-}
\ No newline at end of file
+}
+
+// [[Rcpp::export]]
+void tdf_set_num_threads(const size_t n)
+{
+    ThreadingManager::get_instance().set_num_threads(n);
+}
