@@ -1,6 +1,6 @@
 /*
  *   OpenTIMS: a fully open-source library for opening Bruker's TimsTOF data files.
- *   Copyright (C) 2020 Michał Startek and Mateusz Łącki
+ *   Copyright (C) 2020-2021 Michał Startek and Mateusz Łącki
  *
  *   This program is free software: you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License, version 3 only,
@@ -14,14 +14,19 @@
  *   You should have received a copy of the GNU General Public License
  *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
+
 #include <cstdlib>
 #include <cassert>
 #include <cstdint>
 #include <string>
 #include <cstring>
+#include <atomic>
 #include <vector>
 #include <iostream>
+#include <locale>
 #include <memory>
+#include <limits>
+#include <thread>
 #include <unordered_map>
 
 
@@ -36,7 +41,8 @@
 #endif
 
 #include "tof2mz_converter.h"
-#include "scan2drift_converter.h"
+#include "scan2inv_ion_mobility_converter.h"
+#include "thread_mgr.h"
 
 TimsFrame::TimsFrame(uint32_t _id,
                      uint32_t _num_scans,
@@ -93,8 +99,7 @@ void TimsFrame::print() const
 void TimsFrame::decompress(char* decompression_buffer, ZSTD_DCtx* decomp_ctx)
 {
     uint32_t tims_packet_size = *reinterpret_cast<const uint32_t*>(tims_bin_frame);
-    uint32_t nnum_scans = *(reinterpret_cast<const uint32_t*>(tims_bin_frame)+1);
-    assert(nnum_scans == num_scans);
+    assert(num_scans == *(reinterpret_cast<const uint32_t*>(tims_bin_frame)+1));
 
     size_t dsbytes = data_size_bytes();
 
@@ -108,7 +113,14 @@ void TimsFrame::decompress(char* decompression_buffer, ZSTD_DCtx* decomp_ctx)
     if(decomp_ctx == nullptr)
         decomp_ctx = parent_tdh.zstd_dctx;
 
-    ZSTD_decompressDCtx(decomp_ctx, decompression_buffer, dsbytes, tims_bin_frame + 8, tims_packet_size - 8);
+    size_t dec_result = ZSTD_decompressDCtx(decomp_ctx, decompression_buffer, dsbytes, tims_bin_frame + 8, tims_packet_size - 8);
+    if(ZSTD_isError(dec_result))
+    {
+        std::string err = "Error uncompressing frame, error code: ";
+        err += std::to_string(dec_result);
+        err += ". File is either corrupted, or in a (yet) unsupported variant of the format.";
+        throw std::runtime_error(err);
+    }
 
     size_t dsints = data_size_ints();
     bytes0 = decompression_buffer;
@@ -128,7 +140,7 @@ void TimsFrame::save_to_buffs(uint32_t* frame_ids,
                               uint32_t* tofs,
                               uint32_t* intensities,
                               double* mzs,
-                              double* drift_times,
+                              double* inv_ion_mobilities,
                               double* retention_times,
                               ZSTD_DCtx* decomp_ctx)
 {
@@ -139,7 +151,7 @@ void TimsFrame::save_to_buffs(uint32_t* frame_ids,
     std::unique_ptr<uint32_t[]> tofs_hndl;
     std::unique_ptr<uint32_t[]> intensities_hndl;
 
-    if(scan_ids == nullptr)
+    if(scan_ids == nullptr && inv_ion_mobilities != nullptr)
     {
         scan_ids_hndl = std::make_unique<uint32_t[]>(num_peaks);
         scan_ids = scan_ids_hndl.get();
@@ -175,12 +187,13 @@ void TimsFrame::save_to_buffs(uint32_t* frame_ids,
         const uint32_t no_peaks = back_data(scan_idx+1) / 2;
 
         const uint32_t for_loop_end = no_peaks + peaks_processed;
-        for(uint32_t ii = peaks_processed; ii < for_loop_end; ii++)
-            scan_ids[ii] = scan_idx;
+
+        if(scan_ids != nullptr)
+            for(uint32_t ii = peaks_processed; ii < for_loop_end; ii++)
+                scan_ids[ii] = scan_idx;
 
         for(uint32_t ii = 0; ii < no_peaks; ii++)
         {
-            //scan_ids[peaks_processed] = scan_idx;
             accum_tofs += back_data(read_offset);
             tofs[peaks_processed] = accum_tofs;
             read_offset++;
@@ -194,9 +207,12 @@ void TimsFrame::save_to_buffs(uint32_t* frame_ids,
 
     const uint32_t nnum_peaks = num_peaks;
 
+    if(scan_ids != nullptr)
+        for(uint32_t ii = peaks_processed; ii < nnum_peaks; ii++)
+            scan_ids[ii] = num_scans_m1;
+
     while(peaks_processed < nnum_peaks)
     {
-        scan_ids[peaks_processed] = num_scans_m1;
         accum_tofs += back_data(read_offset);
         tofs[peaks_processed] = accum_tofs;
         read_offset++;
@@ -220,23 +236,14 @@ void TimsFrame::save_to_buffs(uint32_t* frame_ids,
         for(size_t idx = 0; idx < nnum_peaks; idx++)
             retention_times[idx] = time;
 
-    if(drift_times != nullptr)
-        parent_tdh.scan2drift_converter->convert(id, drift_times, scan_ids, nnum_peaks);
+    if(inv_ion_mobilities != nullptr)
+        parent_tdh.scan2inv_ion_mobility_converter->convert(id, inv_ion_mobilities, scan_ids, nnum_peaks);
 
     if(needs_closure)
         close();
 }
 
-
-void Peak::print()
-{
-#ifndef OPENTIMS_BUILDING_R
-    std::cout << frame_id << "  " << scan_id << "   " << tof << "    " << intensity << std::endl;
-#endif
-}
-
-
-int tims_sql_callback(void* out, int cols, char** row, char**)
+int tims_sql_callback(void* out, [[maybe_unused]] int cols, char** row, char**)
 {
     assert(cols == 7);
     assert(row != NULL);
@@ -247,36 +254,84 @@ int tims_sql_callback(void* out, int cols, char** row, char**)
     return 0;
 }
 
+int check_compression(void*, [[maybe_unused]] int cols, char** row, char**)
+{
+    assert(cols == 1);
+    assert(row != NULL);
+    assert(row[0] != NULL);
+    if(atoi(row[0]) != 2)
+    {
+        std::string error_msg = "Compression algorithm used in your TDF dataset: ";
+        error_msg += row[0];
+        error_msg += " is not (yet) supported by OpenTIMS. Right now only algorithm 2 (zstd) is supported.";
+        throw std::runtime_error(error_msg);
+    }
+    return 0;
+}
 
+#ifndef OPENTIMS_BUILDING_R
+namespace{
+class RAIILocaleHelper
+{
+    const std::locale previous_locale;
+ public:
+    RAIILocaleHelper() : previous_locale(std::locale::global(std::locale("C"))) {};
+    ~RAIILocaleHelper() { std::locale::global(previous_locale); };
+};
+
+class RAIISqlite
+{
+    sqlite3* db_conn;
+ public:
+    RAIISqlite(const std::string& tims_tdf_path) : db_conn(nullptr)
+    {
+        if(sqlite3_open_v2(tims_tdf_path.c_str(), &db_conn, SQLITE_OPEN_READONLY, NULL))
+            throw std::runtime_error(std::string("ERROR opening database: " + tims_tdf_path + " SQLite error msg: ") + sqlite3_errmsg(db_conn));
+    }
+    ~RAIISqlite()
+    {
+        if(db_conn != nullptr)
+            sqlite3_close(db_conn);
+    }
+    sqlite3* release_connection() { sqlite3* ret = db_conn; db_conn = nullptr; return ret; }
+    void query(const std::string& sql, int (*callback)(void*,int,char**,char**), void* arg)
+    {
+        char* error = NULL;
+
+        if(sqlite3_exec(db_conn, sql.c_str(), callback, arg, &error) != SQLITE_OK)
+        {
+	    std::string err_msg(std::string("ERROR performing SQL query. SQLite error msg: ") + error);
+	    sqlite3_free(error);
+	    throw std::runtime_error(err_msg);
+        }
+    }
+
+};
+}
+#endif
 
 void TimsDataHandle::read_sql(const std::string& tims_tdf_path)
 {
-    if(sqlite3_open_v2(tims_tdf_path.c_str(), &db_conn, SQLITE_OPEN_READONLY, NULL))
-        throw std::runtime_error(std::string("ERROR opening database: " + tims_tdf_path + " SQLite error msg: ") + sqlite3_errmsg(db_conn));
+#ifndef OPENTIMS_BUILDING_R
+    RAIILocaleHelper locale_guard;
+    RAIISqlite DB(tims_tdf_path);
 
-    const char sql[] = "SELECT Id, NumScans, NumPeaks, MsMsType, AccumulationTime, Time, TimsId from Frames;";
+    const std::string sql = "SELECT Id, NumScans, NumPeaks, MsMsType, AccumulationTime, Time, TimsId from Frames;";
 
-    char* error = NULL;
+    DB.query(sql, tims_sql_callback, this);
+    DB.query("SELECT Value FROM GlobalMetadata WHERE Key == \"TimsCompressionType\";", check_compression, nullptr);
 
-    if(sqlite3_exec(db_conn, sql, tims_sql_callback, this, &error) != SQLITE_OK)
-    {
-        std::string err_msg(std::string("ERROR performing SQL query. SQLite error msg: ") + error);
-        sqlite3_free(error);
-        sqlite3_close(db_conn);
-        throw std::runtime_error(err_msg);
-    }
+    db_conn = DB.release_connection();
 
+#endif
 }
 
 
-TimsDataHandle::TimsDataHandle(const std::string& tims_tdf_bin_path, const std::string& tims_tdf_path, const std::string& tims_data_dir)
-: tims_dir_path(tims_data_dir), tims_data_bin(tims_tdf_bin_path), zstd_dctx(nullptr), db_conn(nullptr)
+void TimsDataHandle::init()
 {
-    read_sql(tims_tdf_path);
-
     _min_frame_id = (std::numeric_limits<uint32_t>::max)();
     _max_frame_id = (std::numeric_limits<uint32_t>::min)();
-    size_t decomp_buffer_size = 0;
+    decomp_buffer_size = 0;
     for(auto it = frame_descs.begin(); it != frame_descs.end(); it++)
     {
         _min_frame_id = (std::min)(_min_frame_id, it->first);
@@ -288,21 +343,102 @@ TimsDataHandle::TimsDataHandle(const std::string& tims_tdf_bin_path, const std::
     zstd_dctx = ZSTD_createDCtx();
 
     tof2mz_converter = DefaultTof2MzConverterFactory::produceDefaultConverterInstance(*this);
-    scan2drift_converter = DefaultScan2DriftConverterFactory::produceDefaultConverterInstance(*this);
+    scan2inv_ion_mobility_converter = DefaultScan2InvIonMobilityConverterFactory::produceDefaultConverterInstance(*this);
 }
 
+TimsDataHandle::TimsDataHandle(const std::string& tims_tdf_bin_path, const std::string& tims_tdf_path, const std::string& tims_data_dir)
+: tims_dir_path(tims_data_dir), tims_data_bin(tims_tdf_bin_path), zstd_dctx(nullptr)
+#ifndef OPENTIMS_BUILDING_R
+, db_conn(nullptr)
+#endif
+{
+#ifndef OPENTIMS_BUILDING_R
+    read_sql(tims_tdf_path);
+#endif
+    init();
+}
 
 TimsDataHandle::TimsDataHandle(const std::string& tims_data_dir)
 : TimsDataHandle(tims_data_dir + "/analysis.tdf_bin", tims_data_dir + "/analysis.tdf", tims_data_dir)
 {}
 
+#ifdef OPENTIMS_BUILDING_R
+
+union braindead_r
+{
+    double as_dbl;
+    int64_t as_int;
+};
+
+template<typename T> std::vector<T> braindead_r_extract_as_int(const SEXP& vec)
+{
+    /* Quick and dirty workaround for R being braindead.
+     * R holds integers as either IntegerVector or as NumericVector depending on size.
+     * Converting a NumericVector to IntegerVector fails silently, giving zeros.
+     * This function forces it to a sensible C++ int type.
+     */
+
+    std::vector<T> res;
+
+    if(Rf_isInteger(vec))
+    {
+        Rcpp::IntegerVector vint(vec);
+        res.reserve(vint.size());
+        for(int ii=0; ii<vint.size(); ii++)
+            res.push_back(vint[ii]);
+        return res;
+    }
+    else
+    {
+        braindead_r converter;
+        Rcpp::NumericVector vnum(vec);
+        res.reserve(vnum.size());
+        for(int ii=0; ii<vnum.size(); ii++)
+        {
+            converter.as_dbl = vnum[ii];
+            res.push_back(converter.as_int);
+        }
+        return res;
+    }
+}
+
+TimsDataHandle::TimsDataHandle(const std::string& tims_data_dir, const Rcpp::List& analysis_tdf) :
+TimsDataHandle(tims_data_dir)
+{
+
+    std::vector<uint32_t> ids = braindead_r_extract_as_int<uint32_t>(analysis_tdf("Id"));
+    std::vector<uint32_t> num_scans = braindead_r_extract_as_int<uint32_t>(analysis_tdf("NumScans"));
+    std::vector<uint32_t> num_peaks = braindead_r_extract_as_int<uint32_t>(analysis_tdf("NumPeaks"));
+    std::vector<uint32_t> msms_type = braindead_r_extract_as_int<uint32_t>(analysis_tdf("MsMsType"));
+    Rcpp::NumericVector accum_time = analysis_tdf("AccumulationTime");
+    Rcpp::NumericVector time = analysis_tdf("Time");
+    std::vector<uint64_t> tims_id = braindead_r_extract_as_int<uint64_t>(analysis_tdf("TimsId"));
+
+    for(size_t ii = 0; ii < ids.size(); ii++)
+    {
+        frame_descs.emplace(ids[ii], TimsFrame(
+                ids[ii],
+                num_scans[ii],
+                num_peaks[ii],
+                msms_type[ii],
+                100.0 / accum_time[ii],
+                time[ii],
+                tims_id[ii] + tims_data_bin.data(),
+                *this));
+    }
+
+    init();
+}
+#endif
 
 TimsDataHandle::~TimsDataHandle()
 {
     if(zstd_dctx != nullptr)
         ZSTD_freeDCtx(zstd_dctx);
+#ifndef OPENTIMS_BUILDING_R
     if(db_conn != nullptr)
         sqlite3_close(db_conn);
+#endif
     // std::cout << "KABOOM!!!" << std::endl; // JUST A TEST: this can be triggered by Python GC with pybind11.
 }
 
@@ -312,7 +448,7 @@ TimsFrame& TimsDataHandle::get_frame(uint32_t frame_no)
     return frame_descs.at(frame_no); 
 }
 
-const std::unordered_map<uint32_t, TimsFrame>& TimsDataHandle::get_frame_descs() const
+std::unordered_map<uint32_t, TimsFrame>& TimsDataHandle::get_frame_descs()
 {
     return frame_descs;
 }
@@ -350,15 +486,17 @@ void TimsDataHandle::set_converter(std::unique_ptr<Tof2MzConverter>&& converter)
         tof2mz_converter = DefaultTof2MzConverterFactory::produceDefaultConverterInstance(*this);
 }
 
-void TimsDataHandle::set_converter(std::unique_ptr<Scan2DriftConverter>&& converter)
+void TimsDataHandle::set_converter(std::unique_ptr<Scan2InvIonMobilityConverter>&& converter)
 {
     if(converter)
-        scan2drift_converter = std::move(converter);
+        scan2inv_ion_mobility_converter = std::move(converter);
     else
-        scan2drift_converter = DefaultScan2DriftConverterFactory::produceDefaultConverterInstance(*this);
+        scan2inv_ion_mobility_converter = DefaultScan2InvIonMobilityConverterFactory::produceDefaultConverterInstance(*this);
 }
 
-void TimsDataHandle::extract_frames(const uint32_t* indexes, size_t no_indexes, uint32_t* result)
+void TimsDataHandle::extract_frames(const uint32_t* indexes,
+                                    size_t no_indexes,
+                                    uint32_t* result)
 {
     size_t no_peaks = no_peaks_in_frames(indexes, no_indexes);
 
@@ -379,7 +517,10 @@ void TimsDataHandle::extract_frames(const uint32_t* indexes, size_t no_indexes,
 }
 
 
-void TimsDataHandle::extract_frames_slice(uint32_t start, uint32_t end, uint32_t step, uint32_t* result)
+void TimsDataHandle::extract_frames_slice(uint32_t start,
+                                          uint32_t end,
+                                          uint32_t step,
+                                          uint32_t* result)
 {
     size_t no_peaks = no_peaks_in_slice(start, end, step);
 
@@ -401,36 +542,53 @@ void TimsDataHandle::extract_frames_slice(uint32_t start, uint32_t end, uint32_t
 
 #define move_ptr(ptr) if(ptr) ptr += n;
 
-void TimsDataHandle::extract_frames(const uint32_t* indexes, size_t no_indexes, uint32_t* frame_ids, uint32_t* scan_ids, uint32_t* tofs, uint32_t* intensities, double* mzs, double* drift_times, double* retention_times)
+void TimsDataHandle::extract_frames(const uint32_t* indexes,
+                                    size_t no_indexes,
+                                    uint32_t* frame_ids,
+                                    uint32_t* scan_ids,
+                                    uint32_t* tofs,
+                                    uint32_t* intensities,
+                                    double* mzs,
+                                    double* inv_ion_mobilities,
+                                    double* retention_times)
 {
     for(size_t ii = 0; ii < no_indexes; ii++)
     {
         TimsFrame& frame = frame_descs.at(indexes[ii]);
         const size_t n = frame.num_peaks;
-        frame_descs.at(indexes[ii]).save_to_buffs(frame_ids, scan_ids, tofs, intensities, mzs, drift_times, retention_times, zstd_dctx);
+        frame_descs.at(indexes[ii]).save_to_buffs(frame_ids, scan_ids, tofs, intensities, mzs, inv_ion_mobilities, retention_times, zstd_dctx);
         move_ptr(frame_ids);
         move_ptr(scan_ids);
         move_ptr(tofs);
         move_ptr(intensities);
         move_ptr(mzs);
-        move_ptr(drift_times);
+        move_ptr(inv_ion_mobilities);
         move_ptr(retention_times);
     }
 }
 
-void TimsDataHandle::extract_frames_slice(uint32_t start, uint32_t end, uint32_t step, uint32_t* frame_ids, uint32_t* scan_ids, uint32_t* tofs, uint32_t* intensities, double* mzs, double* drift_times, double* retention_times)
+void TimsDataHandle::extract_frames_slice(uint32_t start,
+                                          uint32_t end,
+                                          uint32_t step, 
+                                          uint32_t* frame_ids,
+                                          uint32_t* scan_ids,
+                                          uint32_t* tofs,
+                                          uint32_t* intensities,
+                                          double* mzs,
+                                          double* inv_ion_mobilities,
+                                          double* retention_times)
 {
     for(uint32_t ii = start; ii < end; ii += step)
     {
         TimsFrame& frame = frame_descs.at(ii);
         const size_t n = frame.num_peaks;
-        frame_descs.at(ii).save_to_buffs(frame_ids, scan_ids, tofs, intensities, mzs, drift_times, retention_times, zstd_dctx);
+        frame_descs.at(ii).save_to_buffs(frame_ids, scan_ids, tofs, intensities, mzs, inv_ion_mobilities, retention_times, zstd_dctx);
         move_ptr(frame_ids);
         move_ptr(scan_ids);
         move_ptr(tofs);
         move_ptr(intensities);
         move_ptr(mzs);
-        move_ptr(drift_times);
+        move_ptr(inv_ion_mobilities);
         move_ptr(retention_times);
     }
 }
@@ -468,3 +626,56 @@ size_t TimsDataHandle::expose_frame(size_t frame_no)
     frame.save_to_buffs(nullptr, _scan_ids_buffer.get(), _tofs_buffer.get(), _intensities_buffer.get(), nullptr, nullptr, nullptr, zstd_dctx);
     return frame.num_peaks;
 }
+
+void TimsDataHandle::extract_frames(const std::vector<uint32_t>& indexes,
+                                    uint32_t* const * frame_ids,
+                                    uint32_t* const * scan_ids,
+                                    uint32_t* const * tofs,
+                                    uint32_t* const * intensities,
+                                    double* const * mzs,
+                                    double* const * inv_ion_mobilities,
+                                    double* const * retention_times)
+{
+    std::atomic<size_t> current_task(0);
+
+    ThreadingManager::get_instance().set_shared_threading();
+    size_t n_threads = ThreadingManager::get_instance().get_no_opentims_threads();
+
+    std::vector<std::thread> threads;
+    for(size_t ii=0; ii<n_threads; ii++)
+        threads.emplace_back([&](){
+            std::unique_ptr<ZSTD_DCtx, decltype(&ZSTD_freeDCtx)> zstd(ZSTD_createDCtx(), &ZSTD_freeDCtx);
+            std::unique_ptr<char[]> decomp_buffer = std::make_unique<char[]>(decomp_buffer_size);
+            while(true)
+            {
+                size_t my_task = current_task.fetch_add(1);
+                if(my_task < indexes.size())
+                {
+                    TimsFrame& frame = get_frame(indexes[my_task]);
+                    frame.decompress(decomp_buffer.get(), zstd.get());
+                    frame.save_to_buffs(frame_ids[my_task], scan_ids[my_task], tofs[my_task], intensities[my_task], mzs[my_task], inv_ion_mobilities[my_task], retention_times[my_task]);
+                    frame.close();
+                }
+                else
+                    break;
+            }
+        });
+    for (auto& th : threads) th.join();
+    ThreadingManager::get_instance().set_converter_threading();
+}
+
+
+void TimsDataHandle::per_frame_TIC(uint32_t* result)
+{
+    std::unique_ptr<uint32_t[]> intensities = std::make_unique<uint32_t[]>(max_peaks_in_frame());
+
+    for(auto it = frame_descs.begin(); it != frame_descs.end(); it++)
+    {
+        it->second.save_to_buffs(nullptr, nullptr, nullptr, intensities.get(), nullptr, nullptr, nullptr, zstd_dctx);
+        uint32_t acc = 0;
+        const size_t n_peaks = it->second.num_peaks;
+        for(size_t ii = 0; ii < n_peaks; ii++)
+            acc += intensities[ii];
+        result[it->first - 1] = acc;
+    }
+}
